"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateDefinition = void 0;
const utils_1 = require("./utils");
const m = __importStar(require("./modernExtend"));
const zh = __importStar(require("zigbee-herdsman/dist"));
const philips_1 = require("./philips");
function generateSource(device, generatedExtend) {
    const imports = {};
    generatedExtend.forEach((e) => {
        const lib = e.lib ?? 'modernExtend';
        if (!(lib in imports))
            imports[lib] = [];
        imports[lib].push(e.source.split('(')[0]);
    });
    const importsStr = Object.entries(imports)
        .map((e) => `const {${e[1].join(', ')}} = require('zigbee-herdsman-converters/lib/${e[0]}');`).join('\n');
    return `${importsStr}

const definition = {
    zigbeeModel: ['${device.modelID}'],
    model: '${device.modelID}',
    vendor: '${device.manufacturerName}',
    description: 'Automatically generated definition',
    extend: [${generatedExtend.map((e) => e.source).join(', ')}],
};

module.exports = definition;`;
}
async function generateDefinition(device) {
    const generatedExtend = [];
    for (const endpoint of device.endpoints) {
        const usedExtenders = [];
        const addExtenders = async (cluster, extenders) => {
            const extender = extenders.find((e) => e[0].includes(cluster.name));
            if (extender && !usedExtenders.includes(extender)) {
                usedExtenders.push(extender);
                generatedExtend.push(...(await extender[1](endpoint)));
            }
        };
        for (const cluster of endpoint.getInputClusters()) {
            await addExtenders(cluster, inputExtenders);
        }
        for (const cluster of endpoint.getOutputClusters()) {
            await addExtenders(cluster, outputExtenders);
        }
    }
    const definition = {
        zigbeeModel: [device.modelID],
        model: device.modelID ?? '',
        vendor: device.manufacturerName ?? '',
        description: 'Automatically generated definition',
        extend: generatedExtend.map((e) => e.extend),
        generated: true,
    };
    const externalDefinitionSource = generateSource(device, generatedExtend);
    return { externalDefinitionSource, definition };
}
exports.generateDefinition = generateDefinition;
const inputExtenders = [
    [['msTemperatureMeasurement'], async (endpoint) => [{ extend: m.temperature(), source: 'temperature()' }]],
    [['msPressureMeasurement'], async (endpoint) => [{ extend: m.pressure(), source: 'pressure()' }]],
    [['msRelativeHumidity'], async (endpoint) => [{ extend: m.humidity(), source: 'humidity()' }]],
    [['genPowerCfg'], async (endpoint) => [{ extend: m.batteryPercentage(), source: 'batteryPercentage()' }]],
    [['genOnOff', 'lightingColorCtrl'], extenderOnOffLight],
    [['seMetering', 'haElectricalMeasurement'], extenderElectricityMeter],
    [['closuresDoorLock'], extenderLock],
];
const outputExtenders = [
    [['genIdentify'], async (endpoint) => [{ extend: m.identify(), source: 'identify()' }]],
];
async function extenderLock(endpoint) {
    const pinCodeCount = await (0, utils_1.getClusterAttributeValue)(endpoint, 'closuresDoorLock', 'numOfPinUsersSupported');
    return [{ extend: m.lock({ pinCodeCount }), source: `lock({pinCodeCount: ${pinCodeCount}})` }];
}
async function extenderOnOffLight(endpoint) {
    if (endpoint.supportsInputCluster('lightingColorCtrl')) {
        const colorCapabilities = await (0, utils_1.getClusterAttributeValue)(endpoint, 'lightingColorCtrl', 'colorCapabilities');
        const supportsHueSaturation = (colorCapabilities & 1 << 0) > 0;
        const supportsEnhancedHueSaturation = (colorCapabilities & 1 << 1) > 0;
        const supportsColorXY = (colorCapabilities & 1 << 3) > 0;
        const supportsColorTemperature = (colorCapabilities & 1 << 4) > 0;
        const args = {};
        if (supportsColorTemperature) {
            const minColorTemp = await (0, utils_1.getClusterAttributeValue)(endpoint, 'lightingColorCtrl', 'colorTempPhysicalMin');
            const maxColorTemp = await (0, utils_1.getClusterAttributeValue)(endpoint, 'lightingColorCtrl', 'colorTempPhysicalMax');
            args.colorTemp = { range: [minColorTemp, maxColorTemp] };
        }
        if (supportsColorXY) {
            args.color = true;
            if (supportsHueSaturation || supportsEnhancedHueSaturation) {
                args.color = {};
                if (supportsHueSaturation)
                    args.color.modes = ['xy', 'hs'];
                if (supportsEnhancedHueSaturation)
                    args.color.enhancedHue = true;
            }
        }
        const argsStr = JSON.stringify(args);
        if (endpoint.getDevice().manufacturerID === zh.Zcl.ManufacturerCode.Philips) {
            return [{ extend: (0, philips_1.philipsLight)(args), source: `philipsLight(${argsStr})`, lib: 'philips' }];
        }
        else {
            return [{ extend: m.light(args), source: `light(${argsStr})` }];
        }
    }
    else {
        return [{ extend: m.onOff({ powerOnBehavior: false }), source: 'onOff({powerOnBehavior: false})' }];
    }
}
async function extenderElectricityMeter(endpoint) {
    const metering = endpoint.supportsInputCluster('seMetering');
    const electricalMeasurements = endpoint.supportsInputCluster('haElectricalMeasurement');
    const args = {};
    if (!metering || !electricalMeasurements) {
        args.cluster = metering ? 'metering' : 'electrical';
    }
    const argsStr = Object.keys(args).length ? JSON.stringify(args) : '';
    return [{ extend: m.electricityMeter(args), source: `electricityMeter(${argsStr})` }];
}
//# sourceMappingURL=generateDefinition.js.map