"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useIndexOverride = exports.updateToLatest = exports.isUpdateAvailable = exports.getImageMeta = void 0;
const baseurl = 'https://fw.jethome.ru';
const deviceurl = baseurl + '/api/devices/';
const common = __importStar(require("./common"));
const axios = common.getAxios();
const fs_1 = __importDefault(require("fs"));
let overrideIndexFileName = null;
/**
 * Helper functions
 */
async function getIndex(logger, modelId) {
    if (overrideIndexFileName) {
        logger.debug(`JetHomeOTA: Loading override index ${overrideIndexFileName}`);
        if (common.isValidUrl(overrideIndexFileName)) {
            const index = (await axios.get(overrideIndexFileName)).data;
            return index;
        }
        const index = JSON.parse(fs_1.default.readFileSync(overrideIndexFileName, 'utf-8'));
        return index;
    }
    else {
        const index = (await axios.get(deviceurl + modelId + '/info')).data;
        logger.debug(`JetHomeOTA: downloaded index for ${modelId}`);
        return index;
    }
}
async function getImageMeta(current, logger, device) {
    const modelId = device.modelID;
    const imageType = current.imageType;
    logger.debug(`JetHomeOTA: call getImageMeta for ${modelId}`);
    const images = await getIndex(logger, modelId);
    // we need to return the latest_firmware.release.urls.zigbee.ota
    const jetimage = images.latest_firmware.release.images['zigbee.ota'];
    if (!jetimage) {
        throw new Error(`No image available for imageType '${imageType}'`);
    }
    logger.debug('JetHomeOTA: ver: ' + images.latest_firmware.release.version + ' size: ' + jetimage.filesize + ' url: ' + baseurl + jetimage.url);
    return {
        fileVersion: Number(images.latest_firmware.release.version),
        fileSize: jetimage.filesize,
        url: baseurl + jetimage.url,
    };
}
exports.getImageMeta = getImageMeta;
/**
 * Interface implementation
 */
async function isUpdateAvailable(device, logger, requestPayload = null) {
    return common.isUpdateAvailable(device, logger, common.isNewImageAvailable, requestPayload, getImageMeta);
}
exports.isUpdateAvailable = isUpdateAvailable;
async function updateToLatest(device, logger, onProgress) {
    return common.updateToLatest(device, logger, onProgress, common.getNewImage, getImageMeta, common.getFirmwareFile);
}
exports.updateToLatest = updateToLatest;
const useIndexOverride = (indexFileName) => {
    overrideIndexFileName = indexFileName;
};
exports.useIndexOverride = useIndexOverride;
exports.getImageMeta = getImageMeta;
exports.isUpdateAvailable = isUpdateAvailable;
exports.updateToLatest = updateToLatest;
exports.useIndexOverride = exports.useIndexOverride;
//# sourceMappingURL=jethome.js.map